==> zc.ngi-1.0.tar.gz: zc.ngi-1.0/src/zc/ngi/message.py <==
##############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Sample client that sends a single message and waits for a reply

$Id: message.py 69400 2006-08-11 12:55:34Z jim $
"""

import threading

class CouldNotConnect(Exception):
    """Could not connect to a server
    """

class UnexpectedResponse(Exception):
    """Got an unexpected response from a server
    """

class Message:

    def __init__(self, message, expected, notify):
        self.message = message
        self.expected = expected
        self.notify = notify
        self.input = ''

    def connected(self, connection):
        connection.setHandler(self)
        connection.write(self.message)

    def failed_connect(self, reason):
        self.notify(None, reason)

    def handle_input(self, connection, data):
        self.input += data
        if self.expected is not None and self.expected(self.input):
            connection.close()
            self.handle_close(connection)

    def handle_close(self, connection, reason=None):
        self.notify(self.input, reason)


def message(connector, addr, message, expected=None):
    result = []
    lock = threading.Lock()
    lock.acquire()
    def notify(*args):
        if result:
            return # already notified
        result.extend(args)
        lock.release()
    connector(addr, Message(message, expected, notify))
    lock.acquire()
    data, reason = result

    if reason is None:
        return data

    if data is None:
        raise CouldNotConnect(reason)

    if expected is not None and not expected(data):
        raise UnexpectedResponse(data, reason)

    return data

==> zc.ngi-1.1.5.tar.gz: zc.ngi-1.1.5/src/zc/ngi/message.py <==
##############################################################################
#
# Copyright (c) 2006 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Sample client that sends a single message and waits for a reply

$Id: message.py 100261 2009-05-22 22:57:50Z jim $
"""

import threading

class CouldNotConnect(Exception):
    """Could not connect to a server
    """

class UnexpectedResponse(Exception):
    """Got an unexpected response from a server
    """

class Message:

    def __init__(self, message, expected, notify):
        self.message = message
        self.expected = expected
        self.notify = notify
        self.input = ''

    def connected(self, connection):
        connection.setHandler(self)
        connection.write(self.message)

    def failed_connect(self, reason):
        self.notify(None, reason)

    def handle_input(self, connection, data):
        self.input += data
        if self.expected is not None and self.expected(self.input):
            connection.close()
            self.handle_close(connection)

    def handle_close(self, connection, reason=None):
        self.notify(self.input, reason)


def message(connect, addr, message, expected=None):
    result = []
    lock = threading.Lock()
    lock.acquire()
    def notify(*args):
        if result:
            return # already notified
        result.extend(args)
        lock.release()
    connect(addr, Message(message, expected, notify))
    lock.acquire()
    data, reason = result

    if reason is None:
        return data

    if data is None:
        raise CouldNotConnect(reason)

    if expected is not None and not expected(data):
        raise UnexpectedResponse(data, reason)

    return data
